/*
 *  $Id$
 */

/**
 *  @file client_protocol.h
 *
 *  @project btChat (http://www.mulliner.org/bluetooth/btchat/)
 *
 *  @author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 *  (c) Collin R. Mulliner
 *
 */

#ifndef __CLIENT_PROTOCOL_H__
#define __CLIENT_PROTOCOL_H__

enum btcclient_protocol_pkgtype_t {
	BTCCLIENT_PROTOCOL_NULL = 0,
	BTCCLIENT_PROTOCOL_ACCOUNT,
	BTCCLIENT_PROTOCOL_MSG,
	BTCCLIENT_PROTOCOL_TRUST,
	BTCCLIENT_PROTOCOL_LOGIN,
	BTCCLIENT_PROTOCOL_UNKNOWN = 255
};

/**
 *  @brief protocol header
 *
 *  Every protocol event starts with this header.
 *
 */
struct btcclient_protocol_header_t {
	__uint8_t version;  /**< protocol version */
	__uint8_t pkg_type; /**< packet type - see btcclient_protocol_pkgtype_t */
	__uint32_t time;    /**< time at which packet was send (un*x time in seconds) */
	__uint16_t length;  /**< packet length, inclusive headers */
};

#ifndef __BTCHAT_ENUMS__
#define __BTCHAT_ENUMS__

enum btcclient_protocol_trust_level_route_t {
	BTCHAT_TRUST_LEVEL_ROUTE_DONTCONNECT = -1,
	BTCHAT_TRUST_LEVEL_ROUTE_NEUTRAL = 0,
	BTCHAT_TRUST_LEVEL_ROUTE_ALLOW
};

enum btcclient_protocol_trust_level_general_t {
	BTCHAT_TRUST_LEVEL_GENERAL_IGNORE = -1,
	BTCHAT_TRUST_LEVEL_GENERAL_NEUTRAL = 0,
	BTCHAT_TRUST_LEVEL_GENERAL_FRIEND
};

enum btcclient_protocol_discovery_method_t {
	BTCHAT_DISCOVERY_NULL = 0,
	BTCHAT_DISCOVERY_INQUIRY,
	BTCHAT_DISCOVERY_CONNECT,
	BTCHAT_DISCOVERY_NET,
	BTCHAT_DISCOVERY_LAST = 255
};

enum btcclient_protocol_btchat_node_state_t {
	BTCHAT_NODE_STATE_FIRST = 0,
	BTCHAT_NODE_STATE_OFFLINE,   /**< not visible/connected */
	BTCHAT_NODE_STATE_VISIBLE,   /**< seen thru a bluetooth inquiry */
	BTCHAT_NODE_STATE_ONLINE,    /**< somehow connected to the net */
	BTCHAT_NODE_STATE_AWAY,      /**< connected but idle/away */
	BTCHAT_NODE_STATE_LAST = 255
};
#endif

enum btcclient_protocol_btchat_connection_type_t {
	BTCHAT_CONNECTION_TYPE_NOTCONNECTED = 0,
	BTCHAT_CONNECTION_TYPE_INCOMING,
	BTCHAT_CONNECTION_TYPE_OUTGOING,
	BTCHAT_CONNECTION_TYPE_NET
};

/**
 *  @brief account/device/node event
 *
 *
 */
struct btcclient_protocol_account_t {
	__uint8_t addr[6];       /**< device bdaddr */
	__uint32_t device_class; /**< bluetooth device class (0 = unknown) */
	__uint8_t name[250];     /**< bluetooth device name */
	
	__uint8_t discovery_method; /**< method of device discovery */
	
	/* local to remote (set by local user) */
	__int8_t trust_route;   /**< routing trust level */
	__int8_t trust_general; /**< general trust level */

	/* remote to local (set by remote user) */
	__int8_t trusted_route;   /**< routing trust level */
	__int8_t trusted_general; /**< general trust level */
	
	__uint8_t connection; /**< type of connection */
	__uint8_t hops;       /**< min. distance - number of hops */
	__uint8_t state;      /**< status of this node */
};

/**
 *  @brief login
 *
 */
struct btcclient_protocol_login_t {
	__uint8_t name[250]; /**< local name/alias - set by user */
};

/**
 *  @brief a message event
 *
 */
struct btcclient_protocol_msg_t {
	__uint8_t src[6]; /**< source bdaddr */
	__uint8_t dst[6]; /**< destination bdaddr */
	
	/* __uint8_t *msg; */	/**< message bytes */
};

/**
 *  @brief change of trust level
 *
 */
struct btcclient_protocol_trustlevel_t {
	__uint8_t target[6]; /**< device bdaddr */
	__int8_t route;      /**< routing trust level */
	__int8_t general;    /**< general trust level */
};

#endif

/*
 *  $Log$
 */
