/*
 *  $Id$
 */

/**
 *  @file tree.h
 *
 *  @project btChat (http://www.mulliner.org/bluetooth/btchat/)
 *
 *  @author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 *  (c) Collin R. Mulliner
 *
 */

#ifndef __TREE_H__
#define __TREE_H__

struct tree_node_t {	
	struct tree_node_t *p; /**< parent node */
	struct tree_node_t *l; /**< left node, smaller node */
	struct tree_node_t *r; /**< right node, lager node */
	
	void *data;            /**< data pointer */
};

typedef int tree_compare_callback(const void *data1, const void *data2);
typedef int tree_walk_callback(struct tree_node_t *node, void *user);

struct tree_t {
	short int splay;          /**< 1=splay tree, else normal */
	unsigned int num;         /**< number of nodes */
	struct tree_node_t *root; /**< tree root */
	
	tree_compare_callback *compare_function;
	tree_walk_callback *walk_function;
};

#endif

int tree_walk(const struct tree_t *tree, void *user);
int tree_access(struct tree_t *tree, const void *search, struct tree_node_t **result);
int tree_access_insert(struct tree_t *tree, const void *search, struct tree_node_t **result);
int tree_remove(struct tree_t *tree, const void *search, int delete_data);
int tree_reset(struct tree_t *tree, int delete_data);

/*
 *  $Log$
 */
