/*
 *  $Id$
 */
 
/**
 *  @file util.c
 *
 *  @project btChat (http://www.mulliner.org/bluetooth/btchat/)
 *
 *  @author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 *  (c) Collin R. Mulliner
 *
 */

#define _GNU_SOURCE
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <termios.h>
#include <netinet/in.h>
#include <netdb.h>
#include <time.h>
#include <unistd.h>
#include <dirent.h>
#include <assert.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>
#include <bluetooth/rfcomm.h>

#include "tree.h"
#include "btchatd.h"

/**
 *  @brief compare two btchat_nodes for searching the tree
 *
 *  @param d1 pointer to left node
 *  @param d2 pointer to right node
 *
 *  @returns 0 if equal, 1 if d1 > d2 and -1 if d2 > d1
 */
int btchat_tree_cmp(const void *d1, const void *d2)
{
	struct btchat_node_t *a1 = (struct btchat_node_t*) d1;
	struct btchat_node_t *a2 = (struct btchat_node_t*) d2;
	
	#ifdef DEBUG
	assert(a1 != NULL);
	assert(a2 != NULL);
	#endif
	
	return(memcmp(a1->addr.b, a2->addr.b, 6));
}

/**
 *  @brief write n bytes to socket
 *
 *  this is taken from: unix network programming by W.R. Stevens
 *  
 *  @param fd   file descriptor
 *  @param vptr bytes to write out
 *  @param n    number of bytes to write
 *
 *  @returns -1 on failure "n" on success
 */
ssize_t nwrite(int fd, const void *vptr, size_t n)
{
	size_t nleft;
	ssize_t nwritten;
	const char *ptr;
	
	
	ptr = vptr;
	nleft = n;
	while (nleft > 0) {
		if ((nwritten = write(fd, ptr, nleft)) <= 0) {
			if (errno == EINTR) {
				nwritten = 0;
			}
			else {
				return(-1);
			}
		}
		nleft -= nwritten;
		ptr += nwritten;
	}
	return(n);
}

/**
 *  @brief read N bytes from file descriptor
 *
 *  @param fd   file descriptor
 *  @param vptr read buffer
 *  @param n    number of bytes to read
 *
 *  @returns number of bytes read or -1 on error
 */
ssize_t cread(int fd, void *vptr, size_t n)
{
	int result;
	
	
reread:
	if ((result = read(fd, vptr, n)) < 0) {
		if (result == EINTR) {
			result = 0;
			goto reread;
		}
		else {
			return(-1);
		}
	}
	return(result);
}

/*
 *  $Log$
 */
